@@ -1,4 +1,4 @@
-/*	$OpenBSD: io.c,v 1.8 2019/11/29 05:09:50 benno Exp $ */
+/*	$OpenBSD: io.c,v 1.9 2020/09/12 15:46:48 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -25,8 +25,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <openssl/x509.h>
-
 #include "extern.h"
 
 void
