@@ -1,4 +1,4 @@
-/*	$OpenBSD: output.c,v 1.15 2020/05/03 20:24:02 deraadt Exp $ */
+/*	$OpenBSD: output.c,v 1.17 2020/09/12 15:46:48 claudio Exp $ */
 /*
  * Copyright (c) 2019 Theo de Raadt <deraadt@openbsd.org>
  *
@@ -27,8 +27,6 @@
 #include <unistd.h>
 #include <time.h>
 
-#include <openssl/x509v3.h>
-
 #include "extern.h"
 
 char		*outputdir;
@@ -111,7 +109,7 @@ output_createtmp(char *name)
 		err(1, "path too long");
 	fd = mkostemp(output_tmpname, O_CLOEXEC);
 	if (fd == -1)
-		err(1, "mkostemp");
+		err(1, "mkostemp: %s", output_tmpname);
 	(void) fchmod(fd, 0644);
 	f = fdopen(fd, "w");
 	if (f == NULL)
